// Librera de C para el manejo de grficos (principalmente en VGA)


#include <dos.h>
#include <math.h>
#include "msdos.h"

#define BYTE unsigned char
#define WORD unsigned int
#define DWORD unsigned long
#define SVGA1280x1024 0x107     //1280x1024x256
#define SVGA1024x768 0x105      //1024x768x256
#define SVGA800x600 0x103       //800x600x256
#define SVGA640x480 0x101       //640x480x256
#define VGA 0x13	//320x200x256
#define MSDOS 0x03	//80x25x16 (MS-DOS)


// Variables globales necesarias para la funcin LEE_COLOR_PALETA

char R;
char G;
char B;


// Funcin que selecciona un modo de vdeo

void MODO_VIDEO (BYTE modo)
{
 asm MOV AL, modo
 asm MOV AH, 0
 asm INT 10h
}


// Funcin que selecciona un modo de vdeo compatible VESA (SVGA)

void MODO_VESA (WORD modo)
{
 asm MOV AH, 4Fh
 asm MOV AL, 02h
 asm MOV BX, modo
 asm INT 10h
}


// Funcin que lee un pixel de unas coordenadas dadas y devuelve el n
// de dicho color

char LEE_PIXEL (WORD cor_x, WORD cor_y)
{
 BYTE color_pixel;
 asm {
  MOV AX, 0A000h
  MOV ES, AX
  MOV BX, cor_x
  MOV DX, cor_y
  XCHG DH, DL
  MOV DI, DX
  SHR DI, 1
  SHR DI, 1
  ADD DI, DX
  ADD DI, BX
  MOV AL ,ES:[DI]
  MOV color_pixel, AL
 }
 return (color_pixel);
}


// Funcin que pone un pixel en las cordenadas deseadas del color que quieras

void PON_PIXEL (WORD cor_x, WORD cor_y, BYTE color)
{
 asm {
  MOV AX, 0A000h
  MOV ES, AX
  MOV AL, color
  MOV BX, cor_x
  MOV DX, cor_y
  XCHG DH, DL
  MOV DI, DX
  SHR DI, 1
  SHR DI, 1
  ADD DI, DX
  ADD DI, BX
  MOV ES:[DI], AL
 }
}


// Funcin que rellena la pantalla del color deseado (muy rpida)

void RELLENA_PANTALLA (BYTE color)
{
 asm {
  MOV AX, 0A000h
  MOV ES, AX
  MOV DI, 0
  MOV AL, color
  MOV AH, AL
  MOV CX, 32000
  CLD
  REP STOSW
 }
}


// Funcin que cambia los componentes RGB que se le den del color elegido

void COLOR_PALETA (BYTE color, BYTE r, BYTE g, BYTE b)
{
 asm {
 MOV AL, color
 MOV DX, 3C8h
 OUT DX, AL
 INC DX
 MOV AL, r
 OUT DX, AL
 MOV AL, g
 OUT DX, AL
 MOV AL, b
 OUT DX, AL
 }
}


// Funcin que lee los componentes RGB de un color que le sea especificado

void LEE_COLOR_PALETA (BYTE color)
{
 asm MOV AL, color
 asm MOV DX, 3C7h
 asm OUT DX, AL

 R = inportb (0x3c9);
 G = inportb (0x3c9);
 B = inportb (0x3c9);
}


// Funcin que realiza un fundido a negro de los colores actuales (no de la
// paleta, como el FUNDIDO_A_COLOR)

void FUNDIDO_A_NEGRO (void)
{
 register WORD colorct;
 BYTE cont;

 for (cont = 0; cont <= 63; cont ++)
 {
  for (colorct = 0; colorct <= 255; colorct++)
  {
   LEE_COLOR_PALETA (colorct);
   if (R > 0)
   R--;
   if (G > 0)
   G--;
   if (B >0)
   B--;
   COLOR_PALETA (colorct, R, G, B);
  }
 }
}


// Funcin que realiza un fundido a color desde una paleta

void FUNDIDO_COLOR (char buffer_paleta[769])
{
 char r, g, b, k;
 register WORD k2, num;

 for (k = 0; k <= 63; k++)
 {
  num = 0;
  for (k2 = 0; k2 <= 255; k2++)
  {
   num++;
   r = buffer_paleta[num] * k / 64;

   num++;
   g = buffer_paleta[num] * k / 64;

   num++;
   b = buffer_paleta[num] * k / 64;

   COLOR_PALETA (k2, r, g ,b);
  }
 }
}


// Funcin que lee y carga una paleta de colores

void CARGA_PALETA (char *fichero, char *paleta_colores)
{
 char r, g, b;
 register int color, num;
 FILE *paleta;

 paleta = fopen (fichero, "rb");

 fread (paleta_colores, 768, 1, paleta);

 num = 0;
 for (color = 0; color <= 255; color++)
 {
  num++;
  r = paleta_colores[num];

  num++;
  g = paleta_colores[num];

  num++;
  b = paleta_colores[num];

  COLOR_PALETA (color, r, g, b);
 }

 fclose (paleta);
}


// Funcin que actualiza la paleta de colores

void PALETA (char *paleta_colores)
{
 char r, g, b;
 register int color, num;
 FILE *paleta;

 num = 0;
 for (color = 0; color <= 255; color++)
 {
  num++;
  r = paleta_colores[num];

  num++;
  g = paleta_colores[num];

  num++;
  b = paleta_colores[num];

  COLOR_PALETA (color, r, g, b);
 }
}


// Funcin que dibuja un crculo en la posicin deseada con el radio y color
// deseados (Nota: no intentar dibujar un crculo con radio 0)

void CIRCULO (WORD x, WORD y, WORD rad, BYTE col)
{
 register WORD x2, y2;

 float deg = 0.005;

 do {
  x2 = (rad*cos(deg));
  y2 = (rad*sin(deg));

  PON_PIXEL (x+x2, y+y2, col);
  deg = deg+0.005;	// Mayor valor de deg menor calidad y viceversa
  } while (deg<6.3);    // por defecto es 0.005
}


// Funcin que realiza una interpolacin de un pixel dado

void INTERPOLA (register WORD x, register WORD y)
{
 register BYTE a, b, c, d;

 a = LEE_PIXEL (x-1, y);
 b = LEE_PIXEL (x+1, y);
 c = LEE_PIXEL (x, y-1);
 d = LEE_PIXEL (x, y+1);

 PON_PIXEL (x, y, (a+b+c+d)>>2);
}

void ROTA_PALETA (BYTE inicio, BYTE final)
{
 register int i;
 register WORD k;
 BYTE temp[3];

 for (k = inicio; k <= final; k++)
 {
  LEE_COLOR_PALETA (k);

  temp[0] = R;
  temp[1] = G;
  temp[2] = B;
 }
  i = final;

  while (i>=inicio)
  {
   LEE_COLOR_PALETA (i);
    for (k = 0; k <= 10000; k++)
   {}
   COLOR_PALETA(i+1, R, G, B);
   i--;
  }
  COLOR_PALETA (inicio+1, temp[0], temp[1], temp[2]);
}